==> subtitlecomposer-0.4.1.tar.bz2: subtitlecomposer-0.4.1/src/main/useractions/useraction.cpp <==
/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "useraction.h"
#include <player.h>

using namespace SubtitleComposer;

UserAction::UserAction( KAction* action, int enableFlags ):
	m_action( action ),
	m_enableFlags( enableFlags ),
	m_actionEnabled( action->isEnabled() ),
	m_contextEnabled( false ),
	m_ignoreActionEnabledSignal( false )
{
	connect( action, SIGNAL( enabled(bool) ), this, SLOT( onActionEnabled(bool) ) );

	updateEnabledState();
}

int UserAction::enableFlags()
{
	return m_enableFlags;
}

KAction* UserAction::action()
{
	return m_action;
}

bool UserAction::isEnabled()
{
	return m_actionEnabled && m_contextEnabled;
}

void UserAction::setActionEnabled( bool enabled )
{
	if ( m_actionEnabled != enabled )
	{
		m_actionEnabled = enabled;
		updateEnabledState();
	}
}

void UserAction::setContextFlags( int contextFlags )
{
	bool contextEnabled = (contextFlags & m_enableFlags) == m_enableFlags;
	if ( m_contextEnabled != contextEnabled )
	{
		m_contextEnabled = contextEnabled;
		updateEnabledState();
	}
}

void UserAction::updateEnabledState()
{
	m_ignoreActionEnabledSignal = true;

	m_action->setEnabled( m_actionEnabled && m_contextEnabled );

// 	kdDebug() << "setting action " << (m_action->isEnabled() ? "ENABLED":"DISABLED") << endl;

	m_ignoreActionEnabledSignal = false;
}

void UserAction::onActionEnabled( bool enabled )
{
	if ( ! m_ignoreActionEnabledSignal )
	{
		m_actionEnabled = enabled;
		updateEnabledState();
	}
}



UserActionManager::UserActionManager( KActionCollection* actionCollection ):
	m_actionCollection( actionCollection ),
	m_actionSpecs(),
	m_subtitle( 0 ),
	m_linesWidget( 0 ),
	m_player( 0 ),
	m_translationMode( false ),
	m_contextFlags( UserAction::SubClosed|UserAction::SubTrClosed|UserAction::VideoClosed|UserAction::AudioLevelsClosed|UserAction::FullScreenOff )
{
	m_actionSpecs.setAutoDelete( false );
}

KActionCollection* UserActionManager::actionCollection()
{
	return m_actionCollection;
}

void UserActionManager::setActionCollection( KActionCollection* actionCollection )
{
	m_actionCollection = actionCollection;
}

void UserActionManager::addAction( UserAction* actionSpec )
{
	m_actionSpecs.append( actionSpec );

	actionSpec->setContextFlags( m_contextFlags );
}


void UserActionManager::setSubtitle( Subtitle* subtitle )
{
	if ( m_subtitle )
	{
		disconnect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ),
					this, SLOT( onSubtitleLinesChanged() ) );
		disconnect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ),
					this, SLOT( onSubtitleLinesChanged() ) );

		disconnect( m_subtitle, SIGNAL( primaryDirtyStateChanged(bool) ),
					this, SLOT( onPrimaryDirtyStateChanged(bool) ) );
		disconnect( m_subtitle, SIGNAL( secondaryDirtyStateChanged(bool) ),
					this, SLOT( onSecondaryDirtyStateChanged(bool) ) );

		disconnect( &(m_subtitle->actionManager()), SIGNAL( stateChanged() ),
					this, SLOT( onUndoRedoStateChanged() ) );
	}

	m_subtitle = subtitle;

	int newContextFlags = m_contextFlags & ~UserAction::SubtitleMask;

	if ( m_subtitle )
	{
		const ActionManager* actionManager = &(m_subtitle->actionManager());

		connect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ),
				 this, SLOT( onSubtitleLinesChanged() ) );
		connect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ),
				 this, SLOT( onSubtitleLinesChanged() ) );

		connect( m_subtitle, SIGNAL( primaryDirtyStateChanged(bool) ),
				 this, SLOT( onPrimaryDirtyStateChanged(bool) ) );
		connect( m_subtitle, SIGNAL( secondaryDirtyStateChanged(bool) ),
				 this, SLOT( onSecondaryDirtyStateChanged(bool) ) );

		connect( actionManager, SIGNAL( stateChanged() ),
				 this, SLOT( onUndoRedoStateChanged() ) );

		newContextFlags |= UserAction::SubOpened;

		if ( m_subtitle->isPrimaryDirty() )
			newContextFlags |= UserAction::SubPDirty;
		else
			newContextFlags |= UserAction::SubPClean;

		if ( m_translationMode )
		{
			newContextFlags |= UserAction::SubTrOpened;

			if ( m_subtitle->isSecondaryDirty() )
				newContextFlags |= UserAction::SubSDirty;
			else
				newContextFlags |= UserAction::SubSClean;
		}
		else
			newContextFlags |= UserAction::SubTrClosed;

		if ( m_subtitle->linesCount() > 0 )
			newContextFlags |= UserAction::SubHasLine;
		if ( m_subtitle->linesCount() > 1 )
			newContextFlags |= UserAction::SubHasLines;

		if ( actionManager->hasUndo() )
			newContextFlags |= UserAction::SubHasUndo;
		if ( actionManager->hasRedo() )
			newContextFlags |= UserAction::SubHasRedo;
	}
	else
		newContextFlags |= (UserAction::SubClosed|UserAction::SubTrClosed);

	updateActionsContext( newContextFlags );
}


void UserActionManager::onSubtitleLinesChanged()
{
	int newContextFlags = m_contextFlags & ~(UserAction::SubHasLine|UserAction::SubHasLines);

	if ( m_subtitle->linesCount() > 0 )
		newContextFlags |= UserAction::SubHasLine;
	if ( m_subtitle->linesCount() > 1 )
		newContextFlags |= UserAction::SubHasLines;

	updateActionsContext( newContextFlags );
}

void UserActionManager::onPrimaryDirtyStateChanged( bool dirty )
{
	int newContextFlags = m_contextFlags & ~(UserAction::SubPDirty|UserAction::SubPClean);

	if ( dirty )
		newContextFlags |= UserAction::SubPDirty;
	else
		newContextFlags |= UserAction::SubPClean;

	updateActionsContext( newContextFlags );
}

void UserActionManager::onSecondaryDirtyStateChanged( bool dirty )
{
	int newContextFlags = m_contextFlags & ~(UserAction::SubSDirty|UserAction::SubSClean);

	if ( m_translationMode )
	{
		if ( dirty )
			newContextFlags |= UserAction::SubSDirty;
		else
			newContextFlags |= UserAction::SubSClean;
	}

	updateActionsContext( newContextFlags );
}

void UserActionManager::onUndoRedoStateChanged()
{
	int newContextFlags = m_contextFlags & ~(UserAction::SubHasUndo|UserAction::SubHasRedo);

	if ( m_subtitle->actionManager().hasUndo() )
		newContextFlags |= UserAction::SubHasUndo;
	if ( m_subtitle->actionManager().hasRedo() )
		newContextFlags |= UserAction::SubHasRedo;

	updateActionsContext( newContextFlags );
}

void UserActionManager::setLinesWidget( LinesWidget* linesView )
{
	if ( m_linesWidget )
		disconnect( m_linesWidget, SIGNAL( selectionChanged() ),
					this, SLOT( onLinesWidgetSelectionChanged() ) );

	m_linesWidget = linesView;

	int newContextFlags = m_contextFlags & ~UserAction::SelectionMask;

	if ( m_linesWidget )
	{
		connect( m_linesWidget, SIGNAL( selectionChanged() ),
				 this, SLOT( onLinesWidgetSelectionChanged() ) );

		if ( m_linesWidget->firstSelectedIndex() >= 0 )
			newContextFlags |= UserAction::HasSelection;
	}

	updateActionsContext( newContextFlags );
}

void UserActionManager::onLinesWidgetSelectionChanged()
{
	int newContextFlags = m_contextFlags & ~UserAction::SelectionMask;

	if ( m_linesWidget->firstSelectedIndex() >= 0 )
		newContextFlags |= UserAction::HasSelection;

	updateActionsContext( newContextFlags );
}

void UserActionManager::setAudioLevels( AudioLevels* audiolevels )
{
	int newContextFlags = m_contextFlags & ~UserAction::AudioLevelsMask;

	if ( audiolevels )
		newContextFlags |= UserAction::AudioLevelsOpened;
	else
		newContextFlags |= UserAction::AudioLevelsClosed;

	updateActionsContext( newContextFlags );
}

void UserActionManager::setPlayer( Player* player )
{
	if ( m_player )
	{
		disconnect( m_player, SIGNAL( fileOpened(const QString&) ),
					this, SLOT( onPlayerStateChanged() ) );
		disconnect( m_player, SIGNAL( fileClosed() ),
					this, SLOT( onPlayerStateChanged() ) );
		disconnect( m_player, SIGNAL( playing() ),
					this, SLOT( onPlayerStateChanged() ) );
		disconnect( m_player, SIGNAL( paused() ),
					this, SLOT( onPlayerStateChanged() ) );
		disconnect( m_player, SIGNAL( stopped() ),
					this, SLOT( onPlayerStateChanged() ) );
	}

	m_player = player;

	int newContextFlags = m_contextFlags & ~UserAction::VideoMask;

	if ( m_player )
	{
		connect( m_player, SIGNAL( fileOpened(const QString&) ),
				 this, SLOT( onPlayerStateChanged() ) );
		connect( m_player, SIGNAL( fileClosed() ),
				 this, SLOT( onPlayerStateChanged() ) );
		connect( m_player, SIGNAL( playing() ),
				 this, SLOT( onPlayerStateChanged() ) );
		connect( m_player, SIGNAL( paused() ),
				 this, SLOT( onPlayerStateChanged() ) );
		connect( m_player, SIGNAL( stopped() ),
				 this, SLOT( onPlayerStateChanged() ) );

		Player::State state = m_player->state();
		if ( state > Player::Opening )
		{
			newContextFlags |= UserAction::VideoOpened;
			if ( state > Player::Paused )
				newContextFlags |= UserAction::VideoStopped;
			else
				newContextFlags |= UserAction::VideoPlaying;
		}
		else
			newContextFlags |= UserAction::VideoClosed;
	}

	updateActionsContext( newContextFlags );
}

void UserActionManager::onPlayerStateChanged()
{
	int newContextFlags = m_contextFlags & ~UserAction::VideoMask;

	Player::State state = m_player->state();
	if ( state > Player::Opening )
	{
		newContextFlags |= UserAction::VideoOpened;
		if ( state > Player::Paused )
			newContextFlags |= UserAction::VideoStopped;
		else
			newContextFlags |= UserAction::VideoPlaying;
	}
	else
		newContextFlags |= UserAction::VideoClosed;

	updateActionsContext( newContextFlags );
}

void UserActionManager::setTranslationMode( bool translationMode )
{
	m_translationMode = translationMode;

	int newContextFlags = m_contextFlags & ~(UserAction::SubTrClosed|UserAction::SubTrOpened|
											 UserAction::SubSDirty|UserAction::SubSClean);

	if ( m_subtitle && m_translationMode )
	{
		newContextFlags |= UserAction::SubTrOpened;

		if ( m_subtitle->isSecondaryDirty() )
			newContextFlags |= UserAction::SubSDirty;
		else
			newContextFlags |= UserAction::SubSClean;
	}
	else
		newContextFlags |= UserAction::SubTrClosed;

	updateActionsContext( newContextFlags );
}


void UserActionManager::setFullScreenMode( bool fullScreenMode )
{
	int newContextFlags = m_contextFlags & ~UserAction::FullScreenMask;

	if ( fullScreenMode )
		newContextFlags |= UserAction::FullScreenOn;
	else
		newContextFlags |= UserAction::FullScreenOff;

	updateActionsContext( newContextFlags );
}

void UserActionManager::updateActionsContext( int contextFlags )
{
	if ( m_contextFlags != contextFlags )
	{
		m_contextFlags = contextFlags;
		for ( QPtrListIterator<UserAction> it( m_actionSpecs ); it.current(); ++it )
			it.current()->setContextFlags( m_contextFlags );
	}
}

==> subtitlecomposer-0.4.2.tar.bz2: subtitlecomposer-0.4.2/src/main/useractions/useraction.cpp <==
/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "useraction.h"
#include <player.h>
#include <lineswidget.h>
#include <currentlinewidget.h>

using namespace SubtitleComposer;

UserAction::UserAction( KAction* action, int enableFlags ):
	m_action( action ),
	m_enableFlags( enableFlags ),
	m_actionEnabled( action->isEnabled() ),
	m_contextEnabled( false ),
	m_ignoreActionEnabledSignal( false )
{
	connect( action, SIGNAL( enabled(bool) ), this, SLOT( onActionEnabled(bool) ) );

	updateEnabledState();
}

int UserAction::enableFlags()
{
	return m_enableFlags;
}

KAction* UserAction::action()
{
	return m_action;
}

bool UserAction::isEnabled()
{
	return m_actionEnabled && m_contextEnabled;
}

void UserAction::setActionEnabled( bool enabled )
{
	if ( m_actionEnabled != enabled )
	{
		m_actionEnabled = enabled;
		updateEnabledState();
	}
}

void UserAction::setContextFlags( int contextFlags )
{
	bool contextEnabled = (contextFlags & m_enableFlags) == m_enableFlags;
	if ( m_contextEnabled != contextEnabled )
	{
		m_contextEnabled = contextEnabled;
		updateEnabledState();
	}
}

void UserAction::updateEnabledState()
{
	m_ignoreActionEnabledSignal = true;

	m_action->setEnabled( m_actionEnabled && m_contextEnabled );

// 	kdDebug() << "setting action " << (m_action->isEnabled() ? "ENABLED":"DISABLED") << endl;

	m_ignoreActionEnabledSignal = false;
}

void UserAction::onActionEnabled( bool enabled )
{
	if ( ! m_ignoreActionEnabledSignal )
	{
		m_actionEnabled = enabled;
		updateEnabledState();
	}
}

/// USER ACTION MANAGER

UserActionManager::UserActionManager():
	m_actionSpecs(),
	m_subtitle( 0 ),
	m_linesWidget( 0 ),
	m_player( 0 ),
	m_translationMode( false ),
	m_contextFlags(
		UserAction::SubClosed|UserAction::SubTrClosed|
		UserAction::VideoClosed|
		UserAction::AudioLevelsClosed|
		UserAction::FullScreenOff|
		UserAction::CurrentLineFocusOff
	)
{
	m_actionSpecs.setAutoDelete( false );
}

UserActionManager* UserActionManager::instance()
{
	static UserActionManager actionManager;

	return &actionManager;
}

void UserActionManager::addAction( KAction* action, int enableFlags )
{
	addAction( new UserAction( action, enableFlags ) );
}

void UserActionManager::addAction( UserAction* actionSpec )
{
	m_actionSpecs.append( actionSpec );

	actionSpec->setContextFlags( m_contextFlags );
}


void UserActionManager::setSubtitle( Subtitle* subtitle )
{
	if ( m_subtitle )
	{
		disconnect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ),
					this, SLOT( onSubtitleLinesChanged() ) );
		disconnect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ),
					this, SLOT( onSubtitleLinesChanged() ) );

		disconnect( m_subtitle, SIGNAL( primaryDirtyStateChanged(bool) ),
					this, SLOT( onPrimaryDirtyStateChanged(bool) ) );
		disconnect( m_subtitle, SIGNAL( secondaryDirtyStateChanged(bool) ),
					this, SLOT( onSecondaryDirtyStateChanged(bool) ) );

		disconnect( &(m_subtitle->actionManager()), SIGNAL( stateChanged() ),
					this, SLOT( onUndoRedoStateChanged() ) );
	}

	m_subtitle = subtitle;

	int newContextFlags = m_contextFlags & ~UserAction::SubtitleMask;

	if ( m_subtitle )
	{
		const ActionManager* actionManager = &(m_subtitle->actionManager());

		connect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ),
				 this, SLOT( onSubtitleLinesChanged() ) );
		connect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ),
				 this, SLOT( onSubtitleLinesChanged() ) );

		connect( m_subtitle, SIGNAL( primaryDirtyStateChanged(bool) ),
				 this, SLOT( onPrimaryDirtyStateChanged(bool) ) );
		connect( m_subtitle, SIGNAL( secondaryDirtyStateChanged(bool) ),
				 this, SLOT( onSecondaryDirtyStateChanged(bool) ) );

		connect( actionManager, SIGNAL( stateChanged() ),
				 this, SLOT( onUndoRedoStateChanged() ) );

		newContextFlags |= UserAction::SubOpened;

		if ( m_subtitle->isPrimaryDirty() )
			newContextFlags |= UserAction::SubPDirty;
		else
			newContextFlags |= UserAction::SubPClean;

		if ( m_translationMode )
		{
			newContextFlags |= UserAction::SubTrOpened;

			if ( m_subtitle->isSecondaryDirty() )
				newContextFlags |= UserAction::SubSDirty;
			else
				newContextFlags |= UserAction::SubSClean;
		}
		else
			newContextFlags |= (UserAction::SubTrClosed|UserAction::SubSClean);

		if ( m_subtitle->linesCount() > 0 )
			newContextFlags |= UserAction::SubHasLine;
		if ( m_subtitle->linesCount() > 1 )
			newContextFlags |= UserAction::SubHasLines;

		if ( actionManager->hasUndo() )
			newContextFlags |= UserAction::SubHasUndo;
		if ( actionManager->hasRedo() )
			newContextFlags |= UserAction::SubHasRedo;
	}
	else
		newContextFlags |= (UserAction::SubClosed|UserAction::SubPClean|UserAction::SubTrClosed|UserAction::SubSClean);

	updateActionsContext( newContextFlags );
}


void UserActionManager::onSubtitleLinesChanged()
{
	int newContextFlags = m_contextFlags & ~(UserAction::SubHasLine|UserAction::SubHasLines);

	if ( m_subtitle->linesCount() > 0 )
		newContextFlags |= UserAction::SubHasLine;
	if ( m_subtitle->linesCount() > 1 )
		newContextFlags |= UserAction::SubHasLines;

	updateActionsContext( newContextFlags );
}

void UserActionManager::onPrimaryDirtyStateChanged( bool dirty )
{
	int newContextFlags = m_contextFlags & ~(UserAction::SubPDirty|UserAction::SubPClean);

	if ( dirty )
		newContextFlags |= UserAction::SubPDirty;
	else
		newContextFlags |= UserAction::SubPClean;

	updateActionsContext( newContextFlags );
}

void UserActionManager::onSecondaryDirtyStateChanged( bool dirty )
{
	int newContextFlags = m_contextFlags & ~(UserAction::SubSDirty|UserAction::SubSClean);

	if ( m_translationMode )
	{
		if ( dirty )
			newContextFlags |= UserAction::SubSDirty;
		else
			newContextFlags |= UserAction::SubSClean;
	}
	else
		newContextFlags |= UserAction::SubSClean;

	updateActionsContext( newContextFlags );
}

void UserActionManager::onUndoRedoStateChanged()
{
	int newContextFlags = m_contextFlags & ~(UserAction::SubHasUndo|UserAction::SubHasRedo);

	if ( m_subtitle->actionManager().hasUndo() )
		newContextFlags |= UserAction::SubHasUndo;
	if ( m_subtitle->actionManager().hasRedo() )
		newContextFlags |= UserAction::SubHasRedo;

	updateActionsContext( newContextFlags );
}

void UserActionManager::setLinesWidget( LinesWidget* linesView )
{
	if ( m_linesWidget )
		disconnect( m_linesWidget, SIGNAL( selectionChanged() ),
					this, SLOT( onLinesWidgetSelectionChanged() ) );

	m_linesWidget = linesView;

	int newContextFlags = m_contextFlags & ~UserAction::SelectionMask;

	if ( m_linesWidget )
	{
		connect( m_linesWidget, SIGNAL( selectionChanged() ),
				 this, SLOT( onLinesWidgetSelectionChanged() ) );

		if ( m_linesWidget->firstSelectedIndex() >= 0 )
			newContextFlags |= UserAction::HasSelection;
	}

	updateActionsContext( newContextFlags );
}

void UserActionManager::onLinesWidgetSelectionChanged()
{
	int newContextFlags = m_contextFlags & ~UserAction::SelectionMask;

	if ( m_linesWidget->firstSelectedIndex() >= 0 )
		newContextFlags |= UserAction::HasSelection;

	updateActionsContext( newContextFlags );
}

void UserActionManager::setCurrentLineWidget( CurrentLineWidget* currentLineWidget )
{
	if ( m_curLineWidget )
		disconnect( m_curLineWidget, SIGNAL( focusChanged(bool) ),
					this, SLOT( onCurrentLineWidgetFocusChanged(bool) ) );

	m_curLineWidget = currentLineWidget;

	int newContextFlags = m_contextFlags & ~UserAction::CurrentLineFocusMask;

	if ( m_curLineWidget )
	{
		connect( m_curLineWidget, SIGNAL( focusChanged(bool) ),
				 this, SLOT( onCurrentLineWidgetFocusChanged(bool) ) );

		if ( m_curLineWidget->hasFocus() )
			newContextFlags |= UserAction::CurrentLineFocusOn;
		else
			newContextFlags |= UserAction::CurrentLineFocusOff;
	}

	updateActionsContext( newContextFlags );
}

void UserActionManager::onCurrentLineWidgetFocusChanged( bool hasFocus )
{
	int newContextFlags = m_contextFlags & ~UserAction::CurrentLineFocusMask;

	if ( hasFocus )
		newContextFlags |= UserAction::CurrentLineFocusOn;
	else
		newContextFlags |= UserAction::CurrentLineFocusOff;

	updateActionsContext( newContextFlags );
}

void UserActionManager::setAudioLevels( AudioLevels* audiolevels )
{
	int newContextFlags = m_contextFlags & ~UserAction::AudioLevelsMask;

	if ( audiolevels )
		newContextFlags |= UserAction::AudioLevelsOpened;
	else
		newContextFlags |= UserAction::AudioLevelsClosed;

	updateActionsContext( newContextFlags );
}

void UserActionManager::setPlayer( Player* player )
{
	if ( m_player )
	{
		disconnect( m_player, SIGNAL( fileOpened(const QString&) ),
					this, SLOT( onPlayerStateChanged() ) );
		disconnect( m_player, SIGNAL( fileClosed() ),
					this, SLOT( onPlayerStateChanged() ) );
		disconnect( m_player, SIGNAL( playing() ),
					this, SLOT( onPlayerStateChanged() ) );
		disconnect( m_player, SIGNAL( paused() ),
					this, SLOT( onPlayerStateChanged() ) );
		disconnect( m_player, SIGNAL( stopped() ),
					this, SLOT( onPlayerStateChanged() ) );
	}

	m_player = player;

	int newContextFlags = m_contextFlags & ~UserAction::VideoMask;

	if ( m_player )
	{
		connect( m_player, SIGNAL( fileOpened(const QString&) ),
				 this, SLOT( onPlayerStateChanged() ) );
		connect( m_player, SIGNAL( fileClosed() ),
				 this, SLOT( onPlayerStateChanged() ) );
		connect( m_player, SIGNAL( playing() ),
				 this, SLOT( onPlayerStateChanged() ) );
		connect( m_player, SIGNAL( paused() ),
				 this, SLOT( onPlayerStateChanged() ) );
		connect( m_player, SIGNAL( stopped() ),
				 this, SLOT( onPlayerStateChanged() ) );

		Player::State state = m_player->state();
		if ( state > Player::Opening )
		{
			newContextFlags |= UserAction::VideoOpened;
			if ( state > Player::Paused )
				newContextFlags |= UserAction::VideoStopped;
			else
				newContextFlags |= UserAction::VideoPlaying;
		}
		else
			newContextFlags |= UserAction::VideoClosed;
	}

	updateActionsContext( newContextFlags );
}

void UserActionManager::onPlayerStateChanged()
{
	int newContextFlags = m_contextFlags & ~UserAction::VideoMask;

	Player::State state = m_player->state();
	if ( state > Player::Opening )
	{
		newContextFlags |= UserAction::VideoOpened;
		if ( state > Player::Paused )
			newContextFlags |= UserAction::VideoStopped;
		else
			newContextFlags |= UserAction::VideoPlaying;
	}
	else
		newContextFlags |= UserAction::VideoClosed;

	updateActionsContext( newContextFlags );
}

void UserActionManager::setTranslationMode( bool translationMode )
{
	m_translationMode = translationMode;

	int newContextFlags = m_contextFlags & ~(UserAction::SubTrClosed|UserAction::SubTrOpened|
											 UserAction::SubSDirty|UserAction::SubSClean);

	if ( m_subtitle && m_translationMode )
	{
		newContextFlags |= UserAction::SubTrOpened;

		if ( m_subtitle->isSecondaryDirty() )
			newContextFlags |= UserAction::SubSDirty;
		else
			newContextFlags |= UserAction::SubSClean;
	}
	else
		newContextFlags |= (UserAction::SubTrClosed|UserAction::SubSClean);

	updateActionsContext( newContextFlags );
}


void UserActionManager::setFullScreenMode( bool fullScreenMode )
{
	int newContextFlags = m_contextFlags & ~UserAction::FullScreenMask;

	if ( fullScreenMode )
		newContextFlags |= UserAction::FullScreenOn;
	else
		newContextFlags |= UserAction::FullScreenOff;

	updateActionsContext( newContextFlags );
}

void UserActionManager::updateActionsContext( int contextFlags )
{
	if ( m_contextFlags != contextFlags )
	{
		m_contextFlags = contextFlags;
		for ( QPtrListIterator<UserAction> it( m_actionSpecs ); it.current(); ++it )
			it.current()->setContextFlags( m_contextFlags );
	}
}
